@@ -8,6 +8,8 @@ CURRENCY_WORD_MAPPING = {
     "de": {"$": "Dollar", "€": "Euro", "£": "Pfund"},
     "fr": {"$": "dollar", "€": "euro", "£": "livre"},
     "es": {"$": "dólar", "€": "euro", "£": "libra"},
+    "pt": {"$": "dólar", "€": "euro", "£": "libra"},
+    "it": {"$": "dollaro", "€": "euro", "£": "libbra"},
 }
 
 INVALID_CHARS = {
@@ -15,18 +17,27 @@ INVALID_CHARS = {
     "de": r"[^a-zA-ZäöüÄÖÜß]",
     "fr": r"[^a-zA-ZÀ-ÿ]",
     "es": r"[^a-zA-ZÀ-ÿáéíñúüÁÉÍÑÓÚÜ]",
+    "pt": r"[^a-zA-ZÀ-ÿ]",
+    "it": r"[^a-zA-ZÀ-ÿ]",
 }
 
-PERCENT_WORD = {"en": "percent", "de": "Prozent", "fr": "pour cent", "es": "por ciento"}
-AND_WORD = {"en": "and", "de": "und", "fr": "et", "es": "y"}
+PERCENT_WORD = {
+    "en": "percent",
+    "de": "Prozent",
+    "fr": "pour cent",
+    "es": "por ciento",
+    "pt": "por cento",
+    "it": "per cento",
+}
+AND_WORD = {"en": "and", "de": "und", "fr": "et", "es": "y", "pt": "e", "it": "e"}
 
 METRIC_WORD = {
     "en": {
         "km/h": "kilometers per hour",
         "mph": "miles per hour",
         "m/s": "meters per second",
-        "kg": "kilograms",
-        "cm": "centimeters",
+        "kg": "kilogram",
+        "cm": "centimeter",
     },
     "de": {
         "km/h": "Kilometer pro Stunde",
@@ -39,15 +50,29 @@ METRIC_WORD = {
         "km/h": "kilomètres par heure",
         "mph": "miles par heure",
         "m/s": "mètres par seconde",
-        "kg": "kilogrammes",
-        "cm": "centimètres",
+        "kg": "kilogramme",
+        "cm": "centimètre",
     },
     "es": {
         "km/h": "kilómetros por hora",
         "mph": "millas por hora",
         "m/s": "metros por segundo",
-        "kg": "kilogramos",
-        "cm": "centímetros",
+        "kg": "kilogramo",
+        "cm": "centímetro",
+    },
+    "pt": {
+        "km/h": "quilometros por hora",
+        "mph": "milhas por hora",
+        "m/s": "metros por segundo",
+        "kg": "quilograma",
+        "cm": "centímetro",
+    },
+    "it": {
+        "km/h": "chilometri all'ora",
+        "mph": "miglia all'ora",
+        "m/s": "metri al secondo",
+        "kg": "chilogrammo",
+        "cm": "centimetro",
     },
 }
 
@@ -56,9 +81,12 @@ YEAR_PRECEEDING_WORDS = {
     "de": ["im", "jahr", "vor", "ab", "seit"],
     "fr": ["en", "année", "l'année", "avant", "depuis"],
     "es": ["en", "año", "desde"],
+    "pt": ["em", "ano", "antes"],
+    "it": ["nel", "anno", "prima"],
 }
 
 # the dict key is regex
+# in case of list: [sub regex, findall index, replace text for change_list]
 ABBR_WORDS = {
     "en": {
         r"\ba\.m\.": "am",
@@ -77,7 +105,7 @@ ABBR_WORDS = {
         r"\bmisc\.": "miscellaneous",
         r"\be\.g\.": "for example",
         r"\btv\b": "T V",
-        r"\bno\.": "number",
+        r"\b(no\.) (\d+)": (r"number \2", 0, "number"),
     },
     "de": {
         r"\bz\.B\. ": "zum Beispiel",
@@ -95,7 +123,6 @@ ABBR_WORDS = {
         r"\bMWSt\.": "Mehrwertsteuer",
         r"\bPLZ\b": "Postleitzahl",
         r"\bNK\b": "Nebenkosten",
-        r"\bNr\.": "Nummer",
     },
     "fr": {
         r"\bAuj\. ": "aujourd’hui",
@@ -111,6 +138,15 @@ ABBR_WORDS = {
         r"\bDña\.": "doña",
         r"\bS\.": "siècle",
     },
+    "pt": {
+        r"\ba\.C\. ": "antes de cristo",
+        r"\bDr\.": "doutor",
+        r"\bDra\.": "doutora",
+        r"\bEx\.": "exemplo",
+        r"\bSr\.": "senhor",
+        r"\bSra\.": "senhora",
+    },
+    "it": {},
 }
 
 ABBR_SEARCH_FLAG = {
@@ -118,12 +154,14 @@ ABBR_SEARCH_FLAG = {
     "de": 0,
     "fr": re.IGNORECASE,
     "es": re.IGNORECASE,
+    "pt": re.IGNORECASE,
+    "it": re.IGNORECASE,
 }
 
 
 class BaseFormatter:
     def __init__(self, lang="en"):
-        if lang not in ["en", "de", "fr", "es"]:
+        if lang not in ["en", "de", "fr", "es", "pt", "it"]:
             raise Exception("language not implemented")
 
         self.lang = lang
@@ -161,11 +199,14 @@ class TextFormatter(BaseFormatter):
             if int(value) > 1:
                 if self.lang in ["en", "fr"]:
                     name += "s"
-                elif self.lang == "es":
+                elif self.lang in ["es", "pt"]:
                     if symbol == "$":
                         name += "es"
                     else:
                         name += "s"
+                elif self.lang == "it":
+                    if symbol in ["$", "£"]:
+                        name = name[:-1] + "i"
                 # German: no change
             return name, value
         return None
@@ -219,14 +260,7 @@ class TextFormatter(BaseFormatter):
             return word + self.same_case(word, "s")
         return word
 
-    def format_text(self, text):
-        """
-        a whitelisted fashion as opposed to tokenizing
-        """
-        original_text = text
-        if self.__cache.get(original_text):
-            return self.__cache.get(original_text)
-
+    def preprocess_text(self, text):
         text = text.strip()
 
         # remove punctuation
@@ -284,6 +318,23 @@ class TextFormatter(BaseFormatter):
 
         text = "".join(c_list)
 
+        return text
+
+    def word_splitter(self, text):
+        # can be overwritten
+        text = text.replace(".", " ")
+        return [t for t in text.split() if t]
+
+    def format_text(self, text):
+        """
+        a whitelisted fashion as opposed to tokenizing
+        """
+        original_text = text
+        if self.__cache.get(original_text):
+            return self.__cache.get(original_text)
+
+        text = self.preprocess_text(text)
+
         word_list = []
         change_list = {}
 
@@ -291,13 +342,22 @@ class TextFormatter(BaseFormatter):
         # so it not confuses with period and illegal dots
         search_flag = ABBR_SEARCH_FLAG[self.lang]
         for abbr, repl in ABBR_WORDS[self.lang].items():
+            if isinstance(repl, tuple):
+                repl_reg, repl_idx, repl_string = repl
+            else:
+                repl_reg = repl + " "
+                repl_string = repl
+                repl_idx = -1
+
             for found in re.findall(abbr, text, flags=search_flag):
-                change_list[found] = self.wrap_case(repl)
-            text = re.sub(abbr, repl + " ", text, flags=search_flag)
+                if repl_idx == -1:
+                    change_list[found] = self.wrap_case(repl_string)
+                else:
+                    change_list[found[repl_idx]] = self.wrap_case(repl_string)
+            text = re.sub(abbr, repl_reg, text, flags=search_flag)
 
-        text = text.replace(".", " ")
+        text_iter = self.word_splitter(text)
 
-        text_iter = text.split()
         for idx, word in enumerate(text_iter):
             if not word:
                 continue
@@ -469,8 +529,8 @@ class PhoneFormatter(BaseFormatter):
 
         phone_root = self.get_phone_root(phone)
 
-        if self.lang == "en":
-            return phone_root[0] in ["A", "E", "I", "O", "U"]
+        if self.lang in ["en", "es", "pt"]:
+            return phone_root[0].lower() in ["a", "e", "i", "o", "u"]
 
         # sampa common
         # https://en.wikipedia.org/wiki/SAMPA_chart
@@ -505,7 +565,4 @@ class PhoneFormatter(BaseFormatter):
                 "Q",
             ]
 
-        if self.lang == "es":
-            return phone_root[0] in ["a", "e", "i", "o", "u"]
-
         return False
